@@ -17,7 +17,7 @@
 // Copyright (C) 2007 Iñigo Martínez <inigomartinez@gmail.com>
 // Copyright (C) 2008 Brad Hards <bradh@kde.org>
 // Copyright (C) 2008, 2010 Carlos Garcia Campos <carlosgc@gnome.org>
-// Copyright (C) 2009-2013, 2017, 2018, 2021, 2024, 2025 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2009-2013, 2017, 2018, 2021, 2024-2026 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2009, 2010, 2012, 2013 Thomas Freitag <Thomas.Freitag@alfa.de>
 // Copyright (C) 2010 David Benjamin <davidben@mit.edu>
 // Copyright (C) 2010 Christian Feuersänger <cfeuersaenger@googlemail.com>
@@ -26,6 +26,7 @@
 // Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
 // Copyright (C) 2019, 2022, 2024 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright (C) 2019 Volker Krause <vkrause@kde.org>
+// Copyright (C) 2026 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -41,6 +42,7 @@
 #include "Object.h"
 #include "PopplerCache.h"
 
+#include <stack>
 #include <vector>
 
 class GooString;
@@ -133,7 +135,9 @@ public:
 private:
     std::shared_ptr<GfxFont> doLookupFont(const char *name) const;
 
-    std::unique_ptr<GfxFontDict> fonts;
+    Ref fontDictRef;
+    mutable std::unique_ptr<GfxFontDict> fonts;
+    Object fontDict;
     Object xObjDict;
     Object colorSpaceDict;
     Object patternDict;
@@ -158,7 +162,7 @@ public:
 
     // Constructor for a sub-page object.
     Gfx(PDFDoc *docA, OutputDev *outA, Dict *resDict, const PDFRectangle *box, const PDFRectangle *cropBox, bool (*abortCheckCbkA)(void *data) = nullptr, void *abortCheckCbkDataA = nullptr, Gfx *gfxA = nullptr);
-#ifdef USE_CMS
+#if USE_CMS
     void initDisplayProfile();
 #endif
     ~Gfx();
@@ -169,7 +173,13 @@ public:
     XRef *getXRef() { return xref; }
 
     // Interpret a stream or array of streams.
-    void display(Object *obj, bool topLevel = true);
+    enum class DisplayType
+    {
+        TopLevel,
+        Type3Font,
+        Form
+    };
+    void display(Object *obj, DisplayType displayType = DisplayType::TopLevel);
 
     // Display an annotation, given its appearance (a Form XObject),
     // border style, and bounding box (in default user space).
@@ -210,14 +220,17 @@ private:
     GfxResources *res; // resource stack
     int updateLevel;
 
+    std::stack<DisplayType> displayTypes;
+    std::stack<bool> type3FontIsD1;
+
     GfxState *state; // current graphics state
     int stackHeight; // the height of the current graphics stack
     std::vector<int> stateGuards; // a stack of state limits; to guard against unmatched pops
     bool fontChanged; // set if font or text matrix has changed
     GfxClipType clip; // do a clip?
     int ignoreUndef; // current BX/EX nesting level
-    double baseMatrix[6]; // default matrix for most recent
-                          //   page/form/pattern
+    std::array<double, 6> baseMatrix; // default matrix for most recent
+                                      //   page/form/pattern
     int displayDepth;
     bool ocState; // true if drawing is enabled, false if
                   //   disabled
@@ -235,10 +248,10 @@ private:
 
     static const Operator opTab[]; // table of operators
 
-    void go(bool topLevel);
+    void go(DisplayType displayType);
     void execOp(Object *cmd, Object args[], int numArgs);
-    const Operator *findOp(const char *name);
-    bool checkArg(Object *arg, TchkType type);
+    static const Operator *findOp(const char *name);
+    static bool checkArg(Object *arg, TchkType type);
     Goffset getPos();
 
     int bottomGuard();
@@ -347,7 +360,7 @@ private:
 
     // in-line image operators
     void opBeginImage(Object args[], int numArgs);
-    Stream *buildImageStream();
+    std::unique_ptr<Stream> buildImageStream();
     void opImageData(Object args[], int numArgs);
     void opEndImage(Object args[], int numArgs);
 
