@@ -12,6 +12,7 @@
 // Copyright 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 // Copyright (C) 2025 Jonathan Hähne <jonathan.haehne@hotmail.com>
 // Copyright (C) 2025 Arnav V <arnav0872@gmail.com>
+// Copyright (C) 2026 Adam Sampson <ats@offog.org>
 //
 //========================================================================
 
@@ -64,7 +65,7 @@ unsigned int Linearization::getHintsOffset() const
     int hintsOffset;
 
     Object obj1, obj2;
-    if (linDict.isDict() && (obj1 = linDict.dictLookup("H"), obj1.isArray()) && obj1.arrayGetLength() >= 2 && (obj2 = obj1.arrayGet(0), obj2.isInt()) && obj2.getInt() > 0) {
+    if (linDict.isDict() && (obj1 = linDict.dictLookup("H"), obj1.isArrayOfLengthAtLeast(2)) && (obj2 = obj1.arrayGet(0), obj2.isInt()) && obj2.getInt() > 0) {
         hintsOffset = obj2.getInt();
     } else {
         error(errSyntaxWarning, -1, "Hints table offset in linearization table is invalid");
@@ -79,7 +80,7 @@ unsigned int Linearization::getHintsLength() const
     int hintsLength;
 
     Object obj1, obj2;
-    if (linDict.isDict() && (obj1 = linDict.dictLookup("H"), obj1.isArray()) && obj1.arrayGetLength() >= 2 && (obj2 = obj1.arrayGet(1), obj2.isInt()) && obj2.getInt() > 0) {
+    if (linDict.isDict() && (obj1 = linDict.dictLookup("H"), obj1.isArrayOfLengthAtLeast(2)) && (obj2 = obj1.arrayGet(1), obj2.isInt()) && obj2.getInt() > 0) {
         hintsLength = obj2.getInt();
     } else {
         error(errSyntaxWarning, -1, "Hints table length in linearization table is invalid");
@@ -94,7 +95,7 @@ unsigned int Linearization::getHintsOffset2() const
     int hintsOffset2 = 0; // default to 0
 
     Object obj1;
-    if (linDict.isDict() && (obj1 = linDict.dictLookup("H"), obj1.isArray()) && obj1.arrayGetLength() >= 4) {
+    if (linDict.isDict() && (obj1 = linDict.dictLookup("H"), obj1.isArrayOfLengthAtLeast(4))) {
         Object obj2 = obj1.arrayGet(2);
         if (obj2.isInt() && obj2.getInt() > 0) {
             hintsOffset2 = obj2.getInt();
@@ -112,7 +113,7 @@ unsigned int Linearization::getHintsLength2() const
     int hintsLength2 = 0; // default to 0
 
     Object obj1;
-    if (linDict.isDict() && (obj1 = linDict.dictLookup("H"), obj1.isArray()) && obj1.arrayGetLength() >= 4) {
+    if (linDict.isDict() && (obj1 = linDict.dictLookup("H"), obj1.isArrayOfLengthAtLeast(4))) {
         Object obj2 = obj1.arrayGet(3);
         if (obj2.isInt() && obj2.getInt() > 0) {
             hintsLength2 = obj2.getInt();
