@@ -26,7 +26,7 @@
 // Copyright (C) 2019, 2020 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright (C) 2020 Marek Kasik <mkasik@redhat.com>
 // Copyright (C) 2024 Pratham Gandhi <ppg.1382@gmail.com>
-// Copyright (C) 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright (C) 2025, 2026 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -282,7 +282,7 @@ LinkDest::LinkDest(const Array &a)
                 changeZoom = false;
             } else if (obj2.isNum()) {
                 zoom = obj2.getNum();
-                changeZoom = (zoom == 0) ? false : true;
+                changeZoom = zoom != 0;
             } else {
                 error(errSyntaxWarning, -1, "Bad annotation destination position");
                 return;
@@ -753,17 +753,17 @@ LinkJavaScript::~LinkJavaScript() = default;
 
 Object LinkJavaScript::createObject(XRef *xref, const std::string &js)
 {
-    Dict *linkDict = new Dict(xref);
+    auto linkDict = std::make_unique<Dict>(xref);
     linkDict->add("S", Object(objName, "JavaScript"));
     linkDict->add("JS", Object(std::make_unique<GooString>(js)));
 
-    return Object(linkDict);
+    return Object(std::move(linkDict));
 }
 
 //------------------------------------------------------------------------
 // LinkOCGState
 //------------------------------------------------------------------------
-LinkOCGState::LinkOCGState(const Object *obj) : isValid(true)
+LinkOCGState::LinkOCGState(const Object *obj)
 {
     Object obj1 = obj->dictLookup("State");
     if (obj1.isArray()) {
