@@ -15,12 +15,12 @@
 // Copyright 2020 Thorsten Behrens <Thorsten.Behrens@CIB.de>
 // Copyright 2020 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by Technische Universität Dresden
 // Copyright 2021 Theofilos Intzoglou <int.teo@gmail.com>
-// Copyright 2021 Marek Kasik <mkasik@redhat.com>
+// Copyright 2021, 2025, 2026 Marek Kasik <mkasik@redhat.com>
 // Copyright 2022 Erich E. Hoover <erich.e.hoover@gmail.com>
 // Copyright 2023 Tobias Deiminger <tobias.deiminger@posteo.de>
 // Copyright 2023-2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 // Copyright 2023 Ingo Klöcker <kloecker@kde.org>
-// Copyright 2025 Juraj Šarinay <juraj@sarinay.com>
+// Copyright 2025, 2026 Juraj Šarinay <juraj@sarinay.com>
 //
 //========================================================================
 
@@ -30,8 +30,10 @@
 #include "NSSCryptoSignBackend.h"
 #include "goo/gmem.h"
 
+#include <algorithm>
 #include <array>
 #include <optional>
+#include <span>
 #include <vector>
 #include <filesystem>
 
@@ -93,9 +95,7 @@ struct ESSCertIDv2
  */
 struct SigningCertificateV2
 {
-    ESSCertIDv2 **certs;
-
-    SigningCertificateV2() : certs(nullptr) { }
+    ESSCertIDv2 **certs = nullptr;
 };
 
 /**
@@ -150,6 +150,29 @@ const SEC_ASN1Template ESSCertIDv2Template[] = { { SEC_ASN1_SEQUENCE, 0, nullptr
  */
 const SEC_ASN1Template SigningCertificateV2Template[] = { { SEC_ASN1_SEQUENCE, 0, nullptr, sizeof(SigningCertificateV2) }, { SEC_ASN1_SEQUENCE_OF, offsetof(SigningCertificateV2, certs), ESSCertIDv2Template, 0 }, { 0, 0, nullptr, 0 } };
 
+// SEC_ASN1_INLINE | SEC_ASN1_OPTIONAL and SEC_ASN1EncodeItem() do not work well together within NSS.
+// An additional template is necessary to accept attributes without the two optional fields.
+const SEC_ASN1Template ESSCertIDv2DecodingTemplate[] = { { SEC_ASN1_SEQUENCE, 0, nullptr, sizeof(ESSCertIDv2) },
+                                                         { SEC_ASN1_INLINE | SEC_ASN1_OPTIONAL, offsetof(ESSCertIDv2, hashAlgorithm), SEC_ASN1_GET(SECOID_AlgorithmIDTemplate), 0 },
+                                                         { SEC_ASN1_OCTET_STRING, offsetof(ESSCertIDv2, certHash), nullptr, 0 },
+                                                         { SEC_ASN1_INLINE | SEC_ASN1_OPTIONAL | SEC_ASN1_SKIP, offsetof(ESSCertIDv2, issuerSerial), IssuerSerialTemplate, 0 },
+                                                         { 0, 0, nullptr, 0 } };
+
+const SEC_ASN1Template SigningCertificateV2DecodingTemplate[] = { { SEC_ASN1_SEQUENCE, 0, nullptr, sizeof(SigningCertificateV2) },
+                                                                  { SEC_ASN1_SEQUENCE_OF, offsetof(SigningCertificateV2, certs), ESSCertIDv2DecodingTemplate, 0 },
+                                                                  { 0, 0, nullptr, 0 } };
+// policies omitted on purpose. If present, decoding fails and the attribute is considered invalid, as required by ETSI EN 319 122-1 (CAdES).
+
+const SEC_ASN1Template ESSCertIDDecodingTemplate[] = { { SEC_ASN1_SEQUENCE, 0, nullptr, sizeof(ESSCertIDv2) },
+                                                       { SEC_ASN1_OCTET_STRING, offsetof(ESSCertIDv2, certHash), nullptr, 0 },
+                                                       { SEC_ASN1_INLINE | SEC_ASN1_OPTIONAL | SEC_ASN1_SKIP, offsetof(ESSCertIDv2, issuerSerial), IssuerSerialTemplate, 0 },
+                                                       { 0, 0, nullptr, 0 } };
+
+const SEC_ASN1Template SigningCertificateDecodingTemplate[] = { { SEC_ASN1_SEQUENCE, 0, nullptr, sizeof(SigningCertificateV2) },
+                                                                { SEC_ASN1_SEQUENCE_OF, offsetof(SigningCertificateV2, certs), ESSCertIDDecodingTemplate, 0 },
+                                                                { 0, 0, nullptr, 0 } };
+// policies omitted on purpose. If present, decoding fails and the attribute is considered invalid, as required by ETSI EN 319 122-1 (CAdES).
+
 /*
 struct PKIStatusInfo
 {
@@ -241,6 +264,9 @@ static void shutdownNss()
 // 1.2.840.113549.1.9.16.2.47
 constexpr unsigned char OID_SIGNINGCERTIFICATEV2[] { 0x2a, 0x86, 0x48, 0x86, 0xf7, 0x0d, 0x01, 0x09, 0x10, 0x02, 0x2f };
 
+// 1.2.840.113549.1.9.16.2.12
+constexpr unsigned char OID_SIGNINGCERTIFICATE[] { 0x2a, 0x86, 0x48, 0x86, 0xf7, 0x0d, 0x01, 0x09, 0x10, 0x02, 0x0c };
+
 static NSSCMSAttribute *my_NSS_CMSAttributeArray_FindAttrByOidTag(NSSCMSAttribute **attrs, SECOidTag oidtag, PRBool only)
 {
     SECOidData *oid;
@@ -282,6 +308,22 @@ static NSSCMSAttribute *my_NSS_CMSAttributeArray_FindAttrByOidTag(NSSCMSAttribut
     return attr1;
 }
 
+static std::vector<const NSSCMSAttribute *> CMSAttributeArray_FindAttrByOid(NSSCMSAttribute **attrs, std::span<const unsigned char> oid)
+{
+    if (!attrs) {
+        return {};
+    }
+
+    std::vector<const NSSCMSAttribute *> res;
+
+    while (auto *attr = *attrs++) {
+        if (attr->type.data && std::ranges::equal(oid, std::span { attr->type.data, attr->type.len })) {
+            res.push_back(attr);
+        }
+    }
+    return res;
+}
+
 static SECStatus my_NSS_CMSArray_Add(PLArenaPool *poolp, void ***array, void *obj)
 {
     int n = 0;
@@ -440,7 +482,7 @@ std::string NSSSignatureVerification::getSignerName() const
         return {};
     }
 
-    auto signing_cert = NSS_CMSSignerInfo_GetSigningCertificate(CMSSignerInfo, CERT_GetDefaultCertDB());
+    auto *signing_cert = NSS_CMSSignerInfo_GetSigningCertificate(CMSSignerInfo, CERT_GetDefaultCertDB());
     if (!signing_cert) {
         return {};
     }
@@ -460,7 +502,7 @@ std::string NSSSignatureVerification::getSignerSubjectDN() const
     if (!CMSSignerInfo) {
         return {};
     }
-    auto signing_cert = NSS_CMSSignerInfo_GetSigningCertificate(CMSSignerInfo, CERT_GetDefaultCertDB());
+    auto *signing_cert = NSS_CMSSignerInfo_GetSigningCertificate(CMSSignerInfo, CERT_GetDefaultCertDB());
     if (!signing_cert) {
         return {};
     }
@@ -604,27 +646,54 @@ std::unique_ptr<X509CertificateInfo> NSSSignatureCreation::getCertificateInfo()
 
 static std::optional<std::string> getDefaultFirefoxCertDB()
 {
+    std::vector<std::string> firefoxPaths;
+
 #ifdef _WIN32
     const char *env = getenv("APPDATA");
     if (!env) {
         return {};
     }
-    const std::string firefoxPath = std::string(env) + "/Mozilla/Firefox/Profiles/";
+    firefoxPaths.emplace_back(std::string(env) + "/Mozilla/Firefox/Profiles/");
 #else
     const char *env = getenv("HOME");
+    const char *xdg_config_home = getenv("XDG_CONFIG_HOME");
+    const char *legacy = getenv("MOZ_LEGACY_HOME");
     if (!env) {
         return {};
     }
-    const std::string firefoxPath = std::string(env) + "/.mozilla/firefox/";
+    const std::string firefoxLegacyPath = std::string(env) + "/.mozilla/firefox/";
+
+    if (legacy == nullptr || legacy[0] != '1') {
+        if (xdg_config_home != nullptr) {
+            firefoxPaths.emplace_back(std::string(xdg_config_home) + "/mozilla/firefox/");
+        } else {
+            firefoxPaths.emplace_back(std::string(env) + "/.config/mozilla/firefox/");
+        }
+    }
+    firefoxPaths.emplace_back(firefoxLegacyPath);
 #endif
 
     std::error_code ec; // ensures directory_iterator doesn't throw exceptions
-    for (const auto &entry : std::filesystem::directory_iterator { firefoxPath, ec }) {
-        if (entry.is_directory() && entry.path().string().find("default") != std::string::npos) {
-            return entry.path().string();
+    std::optional<std::string> latestDir;
+    std::filesystem::file_time_type latestWriteTime;
+    for (const std::string &firefoxPath : firefoxPaths) {
+        for (const auto &entry : std::filesystem::directory_iterator { firefoxPath, ec }) {
+            if (entry.is_directory() && entry.path().string().find("default") != std::string::npos) {
+                const auto certPath = entry.path() / "cert9.db";
+                if (std::filesystem::exists(certPath, ec) && std::filesystem::is_regular_file(certPath, ec)) {
+                    const auto writeTime = std::filesystem::last_write_time(certPath, ec);
+                    if (!latestDir.has_value() || writeTime > latestWriteTime) {
+                        latestWriteTime = writeTime;
+                        latestDir = entry.path().string();
+                    }
+                }
+            }
+        }
+        if (latestDir.has_value()) {
+            break;
         }
     }
-    return {};
+    return latestDir;
 }
 
 std::string NSSSignatureConfiguration::sNssDir;
@@ -636,7 +705,7 @@ void NSSSignatureConfiguration::setNSSDir(const GooString &nssDir)
 {
     static bool setNssDirCalled = false;
 
-    if (NSS_IsInitialized() && nssDir.getLength() > 0) {
+    if (NSS_IsInitialized() && !nssDir.empty()) {
         error(errInternal, 0, "You need to call setNSSDir before signature validation related operations happen");
         return;
     }
@@ -650,7 +719,7 @@ void NSSSignatureConfiguration::setNSSDir(const GooString &nssDir)
     atexit(shutdownNss);
 
     bool initSuccess = false;
-    if (nssDir.getLength() > 0) {
+    if (!nssDir.empty()) {
         initSuccess = (NSS_Init(nssDir.c_str()) == SECSuccess);
         sNssDir = nssDir.toStr();
     } else {
@@ -691,7 +760,7 @@ void NSSSignatureConfiguration::setNSSPasswordCallback(const std::function<char
     PasswordFunction = f;
 }
 
-NSSSignatureVerification::NSSSignatureVerification(std::vector<unsigned char> &&p7data) : p7(std::move(p7data)), CMSMessage(nullptr), CMSSignedData(nullptr), CMSSignerInfo(nullptr)
+NSSSignatureVerification::NSSSignatureVerification(std::vector<unsigned char> &&p7data, CryptoSign::SignatureType subfilter) : p7(std::move(p7data)), type(subfilter)
 {
     NSSSignatureConfiguration::setNSSDir({});
     CMSitem.data = p7.data();
@@ -705,7 +774,9 @@ NSSSignatureVerification::NSSSignatureVerification(std::vector<unsigned char> &&
             SECItem usedAlgorithm = (*algs)->algorithm;
             auto hashAlgorithm = SECOID_FindOIDTag(&usedAlgorithm);
             HASH_HashType hashType = HASH_GetHashTypeByOidTag(hashAlgorithm);
-            hashContext = HashContext::create(ConvertHashTypeFromNss(hashType));
+            innerHashAlgorithm = ConvertHashTypeFromNss(hashType);
+            auto outerHashAlgorithm = (type == CryptoSign::SignatureType::adbe_pkcs7_sha1) ? HashAlgorithm::Sha1 : innerHashAlgorithm;
+            hashContext = HashContext::create(outerHashAlgorithm);
 
             if (hashContext) {
                 break;
@@ -715,19 +786,52 @@ NSSSignatureVerification::NSSSignatureVerification(std::vector<unsigned char> &&
     }
 }
 
-NSSSignatureCreation::NSSSignatureCreation(const std::string &certNickname, HashAlgorithm digestAlgTag) : hashContext(HashContext::create(digestAlgTag)), signing_cert(nullptr)
+NSSSignatureCreation::NSSSignatureCreation(const std::string &certNickname, HashAlgorithm digestAlgTag) : hashContext(HashContext::create(digestAlgTag))
 {
     NSSSignatureConfiguration::setNSSDir({});
     signing_cert = CERT_FindCertByNickname(CERT_GetDefaultCertDB(), certNickname.c_str());
+
+    if (!signing_cert) {
+        return;
+    }
+
+    auto *chain = CERT_CertChainFromCert(signing_cert, certUsageEmailSigner, PR_FALSE);
+
+    if (!chain) {
+        return;
+    }
+
+    unsigned int chain_size = 0;
+    for (int i = 0; i < chain->len; i++) {
+        chain_size += chain->certs[i].len;
+    }
+    CERT_DestroyCertificateList(chain);
+
+    // fields present in all signatures
+    estimated_size = 300;
+
+    // SignedData.certificates
+    estimated_size += chain_size;
+
+    // SignerInfo.SignatureValue
+    estimated_size += signing_cert->derPublicKey.len;
+    if (SECOID_GetAlgorithmTag(&signing_cert->subjectPublicKeyInfo.algorithm) == SEC_OID_ANSIX962_EC_PUBLIC_KEY) {
+        estimated_size += signing_cert->derPublicKey.len;
+    }
+
+    // SignerInfo.SignerIdentifier.IssuerAndSerialNumber
+    estimated_size += signing_cert->derIssuer.len + signing_cert->serialNumber.len;
+
+    // ESSCertIDv2.IssuerSerial
+    estimated_size += signing_cert->derIssuer.len + signing_cert->serialNumber.len;
 }
 
 HashAlgorithm NSSSignatureVerification::getHashAlgorithm() const
 {
     if (hashContext) {
         return hashContext->getHashAlgorithm();
-    } else {
-        return HashAlgorithm::Unknown;
     }
+    return HashAlgorithm::Unknown;
 }
 
 void NSSSignatureVerification::addData(unsigned char *data_block, int data_len)
@@ -742,6 +846,11 @@ void NSSSignatureCreation::addData(unsigned char *data_block, int data_len)
     hashContext->updateHash(data_block, data_len);
 }
 
+unsigned int NSSSignatureCreation::estimateSize() const
+{
+    return estimated_size;
+}
+
 NSSSignatureCreation::~NSSSignatureCreation()
 {
     if (signing_cert) {
@@ -775,9 +884,8 @@ static NSSCMSMessage *CMS_MessageCreate(SECItem *cms_item)
                                             nullptr, nullptr /*Password callback*/
                                             ,
                                             nullptr, nullptr /*Decrypt callback*/);
-    } else {
-        return nullptr;
     }
+    return nullptr;
 }
 
 static NSSCMSSignedData *CMS_SignedDataCreate(NSSCMSMessage *cms_msg)
@@ -793,7 +901,7 @@ static NSSCMSSignedData *CMS_SignedDataCreate(NSSCMSMessage *cms_msg)
         return nullptr;
     }
 
-    NSSCMSSignedData *signedData = (NSSCMSSignedData *)NSS_CMSContentInfo_GetContent(cinfo);
+    auto *signedData = (NSSCMSSignedData *)NSS_CMSContentInfo_GetContent(cinfo);
     if (!signedData) {
         error(errInternal, 0, "CError in NSS_CMSContentInfo_GetContent()");
         return nullptr;
@@ -811,9 +919,8 @@ static NSSCMSSignedData *CMS_SignedDataCreate(NSSCMSMessage *cms_msg)
             signedData->tempCerts[i] = CERT_NewTempCertificate(CERT_GetDefaultCertDB(), signedData->rawCerts[i], nullptr, 0, 0);
         }
         return signedData;
-    } else {
-        return nullptr;
     }
+    return nullptr;
 }
 
 static NSSCMSSignerInfo *CMS_SignerInfoCreate(NSSCMSSignedData *cms_sig_data)
@@ -822,9 +929,8 @@ static NSSCMSSignerInfo *CMS_SignerInfoCreate(NSSCMSSignedData *cms_sig_data)
     if (!signerInfo) {
         printf("Error in NSS_CMSSignedData_GetSignerInfo()\n");
         return nullptr;
-    } else {
-        return signerInfo;
     }
+    return signerInfo;
 }
 
 static SignatureValidationStatus NSS_SigTranslate(NSSCMSVerificationStatus nss_code)
@@ -867,12 +973,18 @@ SignatureValidationStatus NSSSignatureVerification::validateSignature()
     digest.data = digest_buffer.data();
     digest.len = digest_buffer.size();
 
-    if ((NSS_CMSSignerInfo_GetSigningCertificate(CMSSignerInfo, CERT_GetDefaultCertDB())) == nullptr) {
-        CMSSignerInfo->verificationStatus = NSSCMSVS_SigningCertNotFound;
+    if (!signingCertificateAvailable()) {
+        return SIGNATURE_INVALID;
     }
 
     SECItem *content_info_data = CMSSignedData->contentInfo.content.data;
-    if (content_info_data != nullptr && content_info_data->data != nullptr) {
+    const bool econtent_present = content_info_data && content_info_data->data;
+    const bool signature_type_requires_econtent = type == CryptoSign::SignatureType::adbe_pkcs7_sha1;
+    if (signature_type_requires_econtent != econtent_present) {
+        return SIGNATURE_INVALID;
+    }
+
+    if (type == CryptoSign::SignatureType::adbe_pkcs7_sha1) {
         /*
           This means it's not a detached type signature
           so the digest is contained in SignedData->contentInfo
@@ -881,7 +993,7 @@ SignatureValidationStatus NSSSignatureVerification::validateSignature()
             return SIGNATURE_DIGEST_MISMATCH;
         }
 
-        auto innerHashContext = HashContext::create(hashContext->getHashAlgorithm());
+        auto innerHashContext = HashContext::create(innerHashAlgorithm);
         innerHashContext->updateHash(content_info_data->data, content_info_data->len);
         digest_buffer = innerHashContext->endHash();
         digest.data = digest_buffer.data();
@@ -890,9 +1002,8 @@ SignatureValidationStatus NSSSignatureVerification::validateSignature()
 
     if (NSS_CMSSignerInfo_Verify(CMSSignerInfo, &digest, nullptr) != SECSuccess) {
         return NSS_SigTranslate(CMSSignerInfo->verificationStatus);
-    } else {
-        return SIGNATURE_VALID;
     }
+    return SIGNATURE_VALID;
 }
 
 void NSSSignatureVerification::validateCertificateAsync(std::chrono::system_clock::time_point validation_time, bool ocspRevocationCheck, bool useAIACertFetch, const std::function<void()> &doneCallback)
@@ -935,14 +1046,17 @@ void NSSSignatureVerification::validateCertificateAsync(std::chrono::system_cloc
         inParams[2].type = cert_pi_end;
     }
 
-    CERT_PKIXVerifyCert(cert, certificateUsageEmailSigner, inParams, nullptr, CMSSignerInfo->cmsg->pwfn_arg);
+    int result = 0;
+    if (CERT_PKIXVerifyCert(cert, certificateUsageEmailSigner, inParams, nullptr, CMSSignerInfo->cmsg->pwfn_arg) != SECSuccess) {
+        result = PORT_GetError();
+    }
 
     // Here we are just faking the asynchronousness. It should
     // somehow be the call to CERT_PXIXVerifyCert that would
     // be put in the thread, but I'm not sure about all of the
     // thread safety of nss.
 
-    validationStatus = std::async([result = PORT_GetError(), doneCallback]() {
+    validationStatus = std::async([result, doneCallback]() {
         if (doneCallback) {
             doneCallback();
         }
@@ -982,10 +1096,10 @@ CertificateValidationStatus NSSSignatureVerification::validateCertificateResult(
     return cachedValidationStatus.value();
 }
 
-std::variant<std::vector<unsigned char>, CryptoSign::SigningError> NSSSignatureCreation::signDetached(const std::string &password)
+std::variant<std::vector<unsigned char>, CryptoSign::SigningErrorMessage> NSSSignatureCreation::signDetached(const std::string &password)
 {
     if (!hashContext) {
-        return CryptoSign::SigningError::InternalError;
+        return CryptoSign::SigningErrorMessage { .type = CryptoSign::SigningError::InternalError, .message = ERROR_IN_CODE_LOCATION };
     }
     std::vector<unsigned char> digest_buffer = hashContext->endHash();
     SECItem digest;
@@ -1001,54 +1115,50 @@ std::variant<std::vector<unsigned char>, CryptoSign::SigningError> NSSSignatureC
     };
     std::unique_ptr<NSSCMSMessage, NSSCMSMessageDestroyer> cms_msg { NSS_CMSMessage_Create(nullptr) };
     if (!cms_msg) {
-        return CryptoSign::SigningError::GenericError;
+        return CryptoSign::SigningErrorMessage { .type = CryptoSign::SigningError::GenericError, .message = ERROR_IN_CODE_LOCATION };
     }
 
     NSSCMSSignedData *cms_sd = NSS_CMSSignedData_Create(cms_msg.get());
     if (!cms_sd) {
-        return CryptoSign::SigningError::GenericError;
+        return CryptoSign::SigningErrorMessage { .type = CryptoSign::SigningError::GenericError, .message = ERROR_IN_CODE_LOCATION };
     }
 
     NSSCMSContentInfo *cms_cinfo = NSS_CMSMessage_GetContentInfo(cms_msg.get());
 
     if (NSS_CMSContentInfo_SetContent_SignedData(cms_msg.get(), cms_cinfo, cms_sd) != SECSuccess) {
-        return CryptoSign::SigningError::GenericError;
+        return CryptoSign::SigningErrorMessage { .type = CryptoSign::SigningError::GenericError, .message = ERROR_IN_CODE_LOCATION };
     }
 
     cms_cinfo = NSS_CMSSignedData_GetContentInfo(cms_sd);
 
     // Attach NULL data as detached data
     if (NSS_CMSContentInfo_SetContent_Data(cms_msg.get(), cms_cinfo, nullptr, PR_TRUE) != SECSuccess) {
-        return CryptoSign::SigningError::GenericError;
+        return CryptoSign::SigningErrorMessage { .type = CryptoSign::SigningError::GenericError, .message = ERROR_IN_CODE_LOCATION };
     }
 
     // hardcode SHA256 these days...
     NSSCMSSignerInfo *cms_signer = NSS_CMSSignerInfo_Create(cms_msg.get(), signing_cert, SEC_OID_SHA256);
     if (!cms_signer) {
-        return CryptoSign::SigningError::GenericError;
+        return CryptoSign::SigningErrorMessage { .type = CryptoSign::SigningError::GenericError, .message = ERROR_IN_CODE_LOCATION };
     }
 
     if (NSS_CMSSignerInfo_IncludeCerts(cms_signer, NSSCMSCM_CertChain, certUsageEmailSigner) != SECSuccess) {
-        return CryptoSign::SigningError::GenericError;
-    }
-
-    if (NSS_CMSSignedData_AddCertificate(cms_sd, signing_cert) != SECSuccess) {
-        return CryptoSign::SigningError::GenericError;
+        return CryptoSign::SigningErrorMessage { .type = CryptoSign::SigningError::GenericError, .message = ERROR_IN_CODE_LOCATION };
     }
 
     if (NSS_CMSSignedData_AddSignerInfo(cms_sd, cms_signer) != SECSuccess) {
-        return CryptoSign::SigningError::GenericError;
+        return CryptoSign::SigningErrorMessage { .type = CryptoSign::SigningError::GenericError, .message = ERROR_IN_CODE_LOCATION };
     }
 
     if (NSS_CMSSignedData_SetDigestValue(cms_sd, SEC_OID_SHA256, &digest) != SECSuccess) {
-        return CryptoSign::SigningError::GenericError;
+        return CryptoSign::SigningErrorMessage { .type = CryptoSign::SigningError::GenericError, .message = ERROR_IN_CODE_LOCATION };
     }
 
     struct PLArenaFreeFalse
     {
         void operator()(PLArenaPool *arena) { PORT_FreeArena(arena, PR_FALSE); }
     };
-    std::unique_ptr<PLArenaPool, PLArenaFreeFalse> arena { PORT_NewArena(CryptoSign::maxSupportedSignatureSize) };
+    std::unique_ptr<PLArenaPool, PLArenaFreeFalse> arena { PORT_NewArena(estimated_size) };
 
     // Add the signing certificate as a signed attribute.
     ESSCertIDv2 *aCertIDs[2];
@@ -1063,7 +1173,7 @@ std::variant<std::vector<unsigned char>, CryptoSign::SigningError> NSSSignatureC
     unsigned char certhash[32];
     SECStatus rv = PK11_HashBuf(SEC_OID_SHA256, certhash, signing_cert->derCert.data, signing_cert->derCert.len);
     if (rv != SECSuccess) {
-        return {};
+        return CryptoSign::SigningErrorMessage { .type = CryptoSign::SigningError::GenericError, .message = ERROR_IN_CODE_LOCATION };
     }
 
     aCertHashItem.type = siBuffer;
@@ -1086,7 +1196,7 @@ std::variant<std::vector<unsigned char>, CryptoSign::SigningError> NSSSignatureC
 
     SECItem *pEncodedCertificate = SEC_ASN1EncodeItem(nullptr, nullptr, &aCertificate, SigningCertificateV2Template);
     if (!pEncodedCertificate) {
-        return {};
+        return CryptoSign::SigningErrorMessage { .type = CryptoSign::SigningError::GenericError, .message = ERROR_IN_CODE_LOCATION };
     }
 
     NSSCMSAttribute aAttribute;
@@ -1120,7 +1230,7 @@ std::variant<std::vector<unsigned char>, CryptoSign::SigningError> NSSSignatureC
     aAttribute.encoded = PR_TRUE;
 
     if (my_NSS_CMSSignerInfo_AddAuthAttr(cms_signer, &aAttribute) != SECSuccess) {
-        return CryptoSign::SigningError::GenericError;
+        return CryptoSign::SigningErrorMessage { .type = CryptoSign::SigningError::GenericError, .message = ERROR_IN_CODE_LOCATION };
     }
 
     SECItem cms_output;
@@ -1129,11 +1239,11 @@ std::variant<std::vector<unsigned char>, CryptoSign::SigningError> NSSSignatureC
 
     NSSCMSEncoderContext *cms_ecx = NSS_CMSEncoder_Start(cms_msg.get(), nullptr, nullptr, &cms_output, arena.get(), passwordCallback, password.empty() ? nullptr : const_cast<char *>(password.c_str()), nullptr, nullptr, nullptr, nullptr);
     if (!cms_ecx) {
-        return CryptoSign::SigningError::GenericError;
+        return CryptoSign::SigningErrorMessage { .type = CryptoSign::SigningError::GenericError, .message = ERROR_IN_CODE_LOCATION };
     }
 
     if (NSS_CMSEncoder_Finish(cms_ecx) != SECSuccess) {
-        return CryptoSign::SigningError::GenericError;
+        return CryptoSign::SigningErrorMessage { .type = CryptoSign::SigningError::GenericError, .message = ERROR_IN_CODE_LOCATION };
     }
 
     auto signature = std::vector<unsigned char>(cms_output.data, cms_output.data + cms_output.len);
@@ -1143,6 +1253,74 @@ std::variant<std::vector<unsigned char>, CryptoSign::SigningError> NSSSignatureC
     return signature;
 }
 
+bool NSSSignatureVerification::signingCertificateAvailable()
+{
+    auto *cert = NSS_CMSSignerInfo_GetSigningCertificate(CMSSignerInfo, CERT_GetDefaultCertDB());
+
+    if (!cert) {
+        return false;
+    }
+
+    if (type != CryptoSign::SignatureType::ETSI_CAdES_detached) {
+        return true;
+    }
+
+    const auto ess_signing_cert = CMSAttributeArray_FindAttrByOid(CMSSignerInfo->authAttr, OID_SIGNINGCERTIFICATE);
+    const auto ess_signing_cert_v2 = CMSAttributeArray_FindAttrByOid(CMSSignerInfo->authAttr, OID_SIGNINGCERTIFICATEV2);
+
+    const SEC_ASN1Template *decoding_template {};
+    HashAlgorithm hash_algorithm = HashAlgorithm::Unknown;
+    const NSSCMSAttribute *attr {};
+    bool have_attr_v2 = false;
+
+    if (ess_signing_cert.size() == 1 && ess_signing_cert_v2.empty()) {
+        attr = ess_signing_cert.front();
+        decoding_template = SigningCertificateDecodingTemplate;
+        hash_algorithm = HashAlgorithm::Sha1;
+    } else if (ess_signing_cert.empty() && ess_signing_cert_v2.size() == 1) {
+        attr = ess_signing_cert_v2.front();
+        decoding_template = SigningCertificateV2DecodingTemplate;
+        hash_algorithm = HashAlgorithm::Sha256;
+        have_attr_v2 = true;
+    } else {
+        return false;
+    }
+
+    if (!attr || !attr->values || !*attr->values || *std::next(attr->values)) {
+        return false;
+    }
+
+    SigningCertificateV2 decoded_attr;
+    auto arena_deleter = [](PLArenaPool *arena) { PORT_FreeArena(arena, PR_FALSE); };
+    std::unique_ptr<PLArenaPool, decltype(arena_deleter)> arena { PORT_NewArena(DER_DEFAULT_CHUNKSIZE) };
+
+    if (SEC_ASN1DecodeItem(arena.get(), &decoded_attr, decoding_template, *attr->values) != SECSuccess) {
+        return false;
+    }
+
+    if (!decoded_attr.certs || !*decoded_attr.certs || !decoded_attr.certs[0]->certHash.data) {
+        return false;
+    }
+
+    if (have_attr_v2) {
+        const SECItem *used_algorithm = &decoded_attr.certs[0]->hashAlgorithm.algorithm;
+        if (used_algorithm->data) {
+            auto hashType = HASH_GetHashTypeByOidTag(SECOID_FindOIDTag(used_algorithm));
+            hash_algorithm = ConvertHashTypeFromNss(hashType);
+        }
+    }
+
+    if (hash_algorithm == HashAlgorithm::Unknown) {
+        return false;
+    }
+
+    auto hash_context = HashContext::create(hash_algorithm);
+    hash_context->updateHash(cert->derCert.data, cert->derCert.len);
+    const auto cert_hash = hash_context->endHash();
+
+    return std::ranges::equal(cert_hash, std::span { decoded_attr.certs[0]->certHash.data, decoded_attr.certs[0]->certHash.len });
+}
+
 static char *GetPasswordFunction(PK11SlotInfo *slot, PRBool /*retry*/, void * /*arg*/)
 {
     const char *name = PK11_GetTokenName(slot);
@@ -1162,7 +1340,7 @@ std::unique_ptr<CryptoSign::VerificationInterface> NSSCryptoSignBackend::createV
     case CryptoSign::SignatureType::ETSI_CAdES_detached:
     case CryptoSign::SignatureType::adbe_pkcs7_detached:
     case CryptoSign::SignatureType::adbe_pkcs7_sha1:
-        return std::make_unique<NSSSignatureVerification>(std::move(pkcs7));
+        return std::make_unique<NSSSignatureVerification>(std::move(pkcs7), type);
     }
     return {};
 }
@@ -1231,7 +1409,7 @@ std::vector<unsigned char> HashContext::endHash()
     return digestBuffer;
 }
 
-HashContext::HashContext(HashAlgorithm algorithm, private_tag) : hash_context { HASH_Create(HASH_GetHashTypeByOidTag(ConvertHashAlgorithmToNss(algorithm))) }, digest_alg_tag(algorithm) { }
+HashContext::HashContext(HashAlgorithm algorithm, private_tag /*unused*/) : hash_context { HASH_Create(HASH_GetHashTypeByOidTag(ConvertHashAlgorithmToNss(algorithm))) }, digest_alg_tag(algorithm) { }
 
 std::unique_ptr<HashContext> HashContext::create(HashAlgorithm algorithm)
 {
