@@ -113,6 +113,7 @@ public:
     void addData(unsigned char *data_block, int data_len) final;
     std::variant<std::vector<unsigned char>, CryptoSign::SigningErrorMessage> signDetached(const std::string &password) final;
     CryptoSign::SignatureType signatureType() const final { return CryptoSign::SignatureType::adbe_pkcs7_detached; }
+    unsigned int estimateSize() const final;
 
     NSSSignatureCreation(const NSSSignatureCreation &) = delete;
     NSSSignatureCreation &operator=(const NSSSignatureCreation &) = delete;
@@ -120,6 +121,7 @@ public:
 private:
     std::unique_ptr<HashContext> hashContext;
     CERTCertificate *signing_cert = nullptr;
+    unsigned int estimated_size = CryptoSign::defaultMaxSignatureSize;
 };
 
 class POPPLER_PRIVATE_EXPORT NSSSignatureConfiguration
