@@ -15,7 +15,7 @@
 //
 // Copyright (C) 2007 Julien Rebetez <julienr@svn.gnome.org>
 // Copyright (C) 2008 Kees Cook <kees@outflux.net>
-// Copyright (C) 2008, 2010, 2017-2021, 2023, 2024 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2008, 2010, 2017-2021, 2023-2025 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2009 Jakub Wilk <jwilk@jwilk.net>
 // Copyright (C) 2012 Fabio D'Urso <fabiodurso@hotmail.it>
 // Copyright (C) 2013 Thomas Freitag <Thomas.Freitag@alfa.de>
@@ -26,7 +26,10 @@
 // Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
 // Copyright (C) 2020 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by Technische Universität Dresden
 // Copyright (C) 2023 Oliver Sander <oliver.sander@tu-dresden.de>
-// Copyright (C) 2024, 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright (C) 2024-2026 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright (C) 2025 Jonathan Hähne <jonathan.haehne@hotmail.com>
+// Copyright (C) 2025 Arnav V <arnav0872@gmail.com>
+// Copyright (C) 2026 Adam Sampson <ats@offog.org>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -39,9 +42,8 @@
 #include <cassert>
 #include <set>
 #include <cstdio>
+#include <cstdlib>
 #include <cstring>
-#include <climits>
-#include "goo/gmem.h"
 #include "goo/GooString.h"
 #include "goo/GooLikely.h"
 #include "Error.h"
@@ -49,34 +51,34 @@
 
 #define OBJECT_TYPE_CHECK(wanted_type)                                                                                                                                                                                                         \
     if (unlikely(type != (wanted_type))) {                                                                                                                                                                                                     \
-        error(errInternal, 0,                                                                                                                                                                                                                  \
-              "Call to Object where the object was type {0:d}, "                                                                                                                                                                               \
-              "not the expected type {1:d}",                                                                                                                                                                                                   \
-              type, wanted_type);                                                                                                                                                                                                              \
+        ::error(errInternal, 0,                                                                                                                                                                                                                \
+                "Call to Object where the object was type {0:d}, "                                                                                                                                                                             \
+                "not the expected type {1:d}",                                                                                                                                                                                                 \
+                type, wanted_type);                                                                                                                                                                                                            \
         abort();                                                                                                                                                                                                                               \
     }
 
 #define OBJECT_2TYPES_CHECK(wanted_type1, wanted_type2)                                                                                                                                                                                        \
     if (unlikely(type != (wanted_type1)) && unlikely(type != (wanted_type2))) {                                                                                                                                                                \
-        error(errInternal, 0,                                                                                                                                                                                                                  \
-              "Call to Object where the object was type {0:d}, "                                                                                                                                                                               \
-              "not the expected type {1:d} or {2:d}",                                                                                                                                                                                          \
-              type, wanted_type1, wanted_type2);                                                                                                                                                                                               \
+        ::error(errInternal, 0,                                                                                                                                                                                                                \
+                "Call to Object where the object was type {0:d}, "                                                                                                                                                                             \
+                "not the expected type {1:d} or {2:d}",                                                                                                                                                                                        \
+                type, wanted_type1, wanted_type2);                                                                                                                                                                                             \
         abort();                                                                                                                                                                                                                               \
     }
 
 #define OBJECT_3TYPES_CHECK(wanted_type1, wanted_type2, wanted_type3)                                                                                                                                                                          \
     if (unlikely(type != (wanted_type1)) && unlikely(type != (wanted_type2)) && unlikely(type != (wanted_type3))) {                                                                                                                            \
-        error(errInternal, 0,                                                                                                                                                                                                                  \
-              "Call to Object where the object was type {0:d}, "                                                                                                                                                                               \
-              "not the expected type {1:d}, {2:d} or {3:d}",                                                                                                                                                                                   \
-              type, wanted_type1, wanted_type2, wanted_type3);                                                                                                                                                                                 \
+        ::error(errInternal, 0,                                                                                                                                                                                                                \
+                "Call to Object where the object was type {0:d}, "                                                                                                                                                                             \
+                "not the expected type {1:d}, {2:d} or {3:d}",                                                                                                                                                                                 \
+                type, wanted_type1, wanted_type2, wanted_type3);                                                                                                                                                                               \
         abort();                                                                                                                                                                                                                               \
     }
 
 #define CHECK_NOT_DEAD                                                                                                                                                                                                                         \
     if (unlikely(type == objDead)) {                                                                                                                                                                                                           \
-        error(errInternal, 0, "Call to dead object");                                                                                                                                                                                          \
+        ::error(errInternal, 0, "Call to dead object");                                                                                                                                                                                        \
         abort();                                                                                                                                                                                                                               \
     }
 
@@ -94,7 +96,7 @@ struct Ref
     int num; // object number
     int gen; // generation number
 
-    static constexpr Ref INVALID() { return { -1, -1 }; };
+    static constexpr Ref INVALID() { return { .num = -1, .gen = -1 }; };
 };
 
 inline bool operator==(const Ref lhs, const Ref rhs) noexcept
@@ -207,6 +209,9 @@ constexpr int numObjTypes = 17; // total number of object types
 class POPPLER_PRIVATE_EXPORT Object
 {
 public:
+    static Object null() { return Object(objNull); }
+    static Object eof() { return Object(objEOF); }
+    static Object error() { return Object(objError); }
     Object() : type(objNone) { }
     ~Object() { free(); }
 
@@ -220,7 +225,6 @@ public:
         type = objInt;
         intg = intgA;
     }
-    explicit Object(ObjType typeA) { type = typeA; }
     explicit Object(double realA)
     {
         type = objReal;
@@ -235,43 +239,49 @@ public:
     explicit Object(std::string &&stringA)
     {
         type = objString;
-        string = new GooString(stringA);
+        string = new GooString(std::move(stringA));
     }
+
     Object(ObjType typeA, std::string &&stringA)
     {
         assert(typeA == objHexString);
         type = typeA;
-        string = new GooString(stringA);
+        string = new GooString(std::move(stringA));
     }
-    Object(ObjType typeA, const char *stringA)
+
+    Object(ObjType typeA, const char *v) : Object(typeA, std::string_view(v)) { }
+    Object(ObjType typeA, std::string_view v)
     {
         assert(typeA == objName || typeA == objCmd);
-        assert(stringA);
         type = typeA;
-        cString = copyString(stringA);
+        cString = (char *)std::malloc(v.size() + 1);
+        memcpy(cString, v.data(), v.size());
+        cString[v.size()] = 0;
     }
     explicit Object(long long int64gA)
     {
         type = objInt64;
         int64g = int64gA;
     }
-    explicit Object(Array *arrayA)
+    explicit Object(std::unique_ptr<Array> arrayA)
     {
         assert(arrayA);
         type = objArray;
-        array = arrayA;
+        array = arrayA.release();
     }
-    explicit Object(Dict *dictA)
+    explicit Object(std::unique_ptr<Dict> &&dictA)
     {
         assert(dictA);
         type = objDict;
-        dict = dictA;
+        dict = dictA.release();
     }
-    explicit Object(Stream *streamA)
+    template<typename StreamType>
+        requires(std::is_base_of_v<Stream, StreamType>)
+    explicit Object(std::unique_ptr<StreamType> &&streamA)
     {
         assert(streamA);
         type = objStream;
-        stream = streamA;
+        stream = streamA.release();
     }
     explicit Object(const Ref r)
     {
@@ -418,9 +428,11 @@ public:
     }
 
     // Special type checking.
-    bool isName(const char *nameA) const { return type == objName && !strcmp(cString, nameA); }
-    bool isDict(const char *dictType) const;
-    bool isCmd(const char *cmdA) const { return type == objCmd && !strcmp(cString, cmdA); }
+    bool isName(std::string_view nameA) const { return type == objName && getName() == nameA; }
+    bool isArrayOfLength(int length) const;
+    bool isArrayOfLengthAtLeast(int length) const;
+    bool isDict(std::string_view dictType) const;
+    bool isCmd(std::string_view cmdA) const { return type == objCmd && cString == cmdA; }
 
     // Accessors.
     bool getBool() const
@@ -459,6 +471,15 @@ public:
         OBJECT_TYPE_CHECK(objString);
         return string;
     }
+    std::unique_ptr<GooString> takeString()
+    {
+        OBJECT_TYPE_CHECK(objString);
+        std::unique_ptr<GooString> str(string);
+        string = nullptr;
+        type = objNull;
+        return str;
+    }
+
     const GooString *getHexString() const
     {
         OBJECT_TYPE_CHECK(objHexString);
@@ -469,6 +490,11 @@ public:
         OBJECT_TYPE_CHECK(objName);
         return cString;
     }
+    std::string getNameString() const
+    {
+        OBJECT_TYPE_CHECK(objName);
+        return std::string { cString };
+    }
     Array *getArray() const
     {
         OBJECT_TYPE_CHECK(objArray);
@@ -524,19 +550,18 @@ public:
 
     // Dict accessors.
     int dictGetLength() const;
-    void dictAdd(char *key, Object &&val) = delete;
-    void dictAdd(const char *key, Object &&val);
-    void dictSet(const char *key, Object &&val);
-    void dictRemove(const char *key);
-    bool dictIs(const char *dictType) const;
-    Object dictLookup(const char *key, int recursion = 0) const;
-    const Object &dictLookupNF(const char *key) const;
     const char *dictGetKey(int i) const;
+    void dictAdd(std::string_view key, Object &&val);
+    void dictSet(std::string_view key, Object &&val);
+    void dictRemove(std::string_view key);
+    bool dictIs(std::string_view dictType) const;
+    Object dictLookup(std::string_view key, int recursion = 0) const;
+    const Object &dictLookupNF(std::string_view key) const;
     Object dictGetVal(int i) const;
     const Object &dictGetValNF(int i) const;
 
     // Stream accessors.
-    [[nodiscard]] bool streamReset();
+    [[nodiscard]] bool streamRewind();
     void streamClose();
     int streamGetChar();
     int streamGetChars(int nChars, unsigned char *buffer);
@@ -558,6 +583,7 @@ public:
     bool getBoolWithDefaultValue(bool defaultValue) const { return (type == objBool) ? booln : defaultValue; }
 
 private:
+    explicit Object(ObjType typeA) { type = typeA; }
     // Free object contents.
     void free();
 
