@@ -20,14 +20,16 @@
 // Copyright (C) 2007 Julien Rebetez <julienr@svn.gnome.org>
 // Copyright (C) 2008 Iñigo Martínez <inigomartinez@gmail.com>
 // Copyright (C) 2012 Fabio D'Urso <fabiodurso@hotmail.it>
-// Copyright (C) 2012, 2017, 2018, 2020, 2021, 2023 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2012, 2017, 2018, 2020, 2021, 2023, 2025 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2013 Thomas Freitag <Thomas.Freitag@alfa.de>
 // Copyright (C) 2013, 2017, 2023 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
 // Copyright (C) 2020 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright (C) 2020, 2021 Nelson Benítez León <nbenitezl@gmail.com>
 // Copyright (C) 2024 Pablo Correa Gómez <ablocorrea@hotmail.com>
-// Copyright (C) 2024, 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright (C) 2024-2026 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright (C) 2025 Even Rouault <even.rouault@spatialys.com>
+// Copyright (C) 2025 Stefan Brüns <stefan.bruens@rwth-aachen.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -40,8 +42,8 @@
 #include <memory>
 #include <mutex>
 
-#include "poppler-config.h"
 #include "Object.h"
+#include "PDFRectangle.h"
 #include "poppler_private_export.h"
 
 class Dict;
@@ -57,28 +59,6 @@ class FormPageWidgets;
 class Form;
 class FormField;
 
-//------------------------------------------------------------------------
-
-class PDFRectangle
-{
-public:
-    double x1, y1, x2, y2;
-
-    PDFRectangle() { x1 = y1 = x2 = y2 = 0; }
-    PDFRectangle(double x1A, double y1A, double x2A, double y2A)
-    {
-        x1 = x1A;
-        y1 = y1A;
-        x2 = x2A;
-        y2 = y2A;
-    }
-    bool isValid() const { return x1 != 0 || y1 != 0 || x2 != 0 || y2 != 0; }
-    bool contains(double x, double y) const { return x1 <= x && x <= x2 && y1 <= y && y <= y2; }
-    void clipTo(PDFRectangle *rect);
-
-    bool operator==(const PDFRectangle &rect) const { return x1 == rect.x1 && y1 == rect.y1 && x2 == rect.x2 && y2 == rect.y2; }
-};
-
 //------------------------------------------------------------------------
 // PageAttrs
 //------------------------------------------------------------------------
@@ -116,7 +96,7 @@ public:
     void clipBoxes();
 
 private:
-    bool readBox(Dict *dict, const char *key, PDFRectangle *box);
+    static bool readBox(Dict *dict, const char *key, PDFRectangle *box);
 
     PDFRectangle mediaBox;
     PDFRectangle cropBox;
@@ -142,7 +122,7 @@ class POPPLER_PRIVATE_EXPORT Page
 {
 public:
     // Constructor.
-    Page(PDFDoc *docA, int numA, Object &&pageDict, Ref pageRefA, std::unique_ptr<PageAttrs> attrsA, Form *form);
+    Page(PDFDoc *docA, int numA, Object &&pageDict, Ref pageRefA, std::unique_ptr<PageAttrs> attrsA);
 
     // Destructor.
     ~Page();
@@ -175,17 +155,20 @@ public:
     PDFDoc *getDoc() { return doc; }
     Ref getRef() { return pageRef; }
 
+    // Keep in API. This is used by GDAL
+    const Object &getPageObj() const { return pageObj; }
+
     // Get resource dictionary.
     Dict *getResourceDict();
     Object *getResourceDictObject();
-    Dict *getResourceDictCopy(XRef *xrefA);
+    std::unique_ptr<Dict> getResourceDictCopy(XRef *xrefA);
 
     // Get annotations array.
     Object getAnnotsObject(XRef *xrefA = nullptr) { return annotsObj.fetch(xrefA ? xrefA : xref); }
     // Add a new annotation to the page
-    bool addAnnot(Annot *annot);
+    bool addAnnot(const std::shared_ptr<Annot> &annot);
     // Remove an existing annotation from the page
-    void removeAnnot(Annot *annot);
+    void removeAnnot(const std::shared_ptr<Annot> &annot);
 
     // Return a list of links.
     std::unique_ptr<Links> getLinks();
@@ -209,7 +192,7 @@ public:
     // Get duration, the maximum length of time, in seconds,
     // that the page is displayed before the presentation automatically
     // advances to the next page
-    double getDuration() { return duration; }
+    double getDuration() const { return duration; }
 
     // Get actions
     Object getActions() { return actions.fetch(xref); }
@@ -235,12 +218,12 @@ public:
 
     void display(Gfx *gfx);
 
-    void makeBox(double hDPI, double vDPI, int rotate, bool useMediaBox, bool upsideDown, double sliceX, double sliceY, double sliceW, double sliceH, PDFRectangle *box, bool *crop);
+    void makeBox(double hDPI, double vDPI, int rotate, bool useMediaBox, bool upsideDown, double sliceX, double sliceY, double sliceW, double sliceH, PDFRectangle *box, bool *crop) const;
 
     void processLinks(OutputDev *out);
 
     // Get the page's default CTM.
-    void getDefaultCTM(double *ctm, double hDPI, double vDPI, int rotate, bool useMediaBox, bool upsideDown);
+    void getDefaultCTM(double *ctm, double hDPI, double vDPI, int rotate, bool useMediaBox, bool upsideDown) const;
 
     bool hasStandaloneFields() const { return !standaloneFields.empty(); }
 
@@ -258,7 +241,7 @@ private:
     const Ref pageRef; // page reference
     int num; // page number
     std::unique_ptr<PageAttrs> attrs; // page attributes
-    Annots *annots; // annotations
+    std::unique_ptr<Annots> annots; // annotations
     Object annotsObj; // annotations array
     Object contents; // page contents
     Object thumb; // page thumbnail
@@ -274,7 +257,7 @@ private:
     // create standalone FormFields to contain those special FormWidgets, as
     // they are 'de facto' being used to implement tooltips. See #34
     std::vector<std::unique_ptr<FormField>> standaloneFields;
-    void loadStandaloneFields(Annots *annotations, Form *form);
+    void loadStandaloneFields(Form *form);
 };
 
 #endif
