@@ -13,7 +13,7 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2006, 2009, 201, 2010, 2013, 2014, 2017-2020, 2025 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2006, 2009, 201, 2010, 2013, 2014, 2017-2020, 2025, 2026 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2006 Krzysztof Kowalczyk <kkowalczyk@gmail.com>
 // Copyright (C) 2009 Ilya Gorenbein <igorenbein@finjan.com>
 // Copyright (C) 2012 Hib Eris <hib@hiberis.nl>
@@ -23,7 +23,7 @@
 // Copyright (C) 2018, 2019 Adam Reichold <adam.reichold@t-online.de>
 // Copyright (C) 2018 Marek Kasik <mkasik@redhat.com>
 // Copyright (C) 2024 Nelson Benítez León <nbenitezl@gmail.com>
-// Copyright (C) 2024, 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright (C) 2024-2026 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 // Copyright (C) 2025 Arnav V <arnav0872@gmail.com>
 //
 // To see a description of the changes please see the Changelog file that
@@ -33,6 +33,7 @@
 
 #include <config.h>
 
+#include <climits>
 #include "Object.h"
 #include "Array.h"
 #include "Dict.h"
@@ -44,7 +45,7 @@
 // Max number of nested objects.  This is used to catch infinite loops
 // in the object structure. And also technically valid files with
 // lots of nested arrays that made us consume all the stack
-#define recursionLimit 500
+constexpr int recursionLimit = 500;
 
 Parser::Parser(XRef *xrefA, std::unique_ptr<Stream> &&streamA, bool allowStreamsA) : lexer { xrefA, std::move(streamA) }
 {
@@ -71,14 +72,14 @@ Object Parser::getObj(int recursion)
 
 static std::unique_ptr<GooString> decryptedString(const GooString *s, const unsigned char *fileKey, CryptAlgorithm encAlgorithm, int keyLength, int objNum, int objGen)
 {
-    DecryptStream decrypt(new MemStream(s->c_str(), 0, s->size(), Object::null()), fileKey, encAlgorithm, keyLength, { objNum, objGen });
+    DecryptStream decrypt(std::make_unique<MemStream>(s->c_str(), 0, s->size(), Object::null()), fileKey, encAlgorithm, keyLength, { .num = objNum, .gen = objGen });
     if (!decrypt.rewind()) {
         return {};
     }
     std::unique_ptr<GooString> res = std::make_unique<GooString>();
     int c;
     while ((c = decrypt.getChar()) != EOF) {
-        res->append((char)c);
+        res->push_back((char)c);
     }
     return res;
 }
@@ -86,7 +87,6 @@ static std::unique_ptr<GooString> decryptedString(const GooString *s, const unsi
 Object Parser::getObj(bool simpleOnly, const unsigned char *fileKey, CryptAlgorithm encAlgorithm, int keyLength, int objNum, int objGen, int recursion, bool strict, bool decryptString)
 {
     Object obj;
-    Stream *str;
 
     // refill buffer after inline image data
     if (inlineImg == 2) {
@@ -102,7 +102,7 @@ Object Parser::getObj(bool simpleOnly, const unsigned char *fileKey, CryptAlgori
     // array
     if (!simpleOnly && buf1.isCmd("[")) {
         shift();
-        obj = Object(new Array(lexer.getXRef()));
+        obj = Object(std::make_unique<Array>(lexer.getXRef()));
         while (!buf1.isCmd("]") && !buf1.isEOF() && recursion + 1 < recursionLimit) {
             Object obj2 = getObj(false, fileKey, encAlgorithm, keyLength, objNum, objGen, recursion + 1);
             obj.arrayAdd(std::move(obj2));
@@ -121,7 +121,7 @@ Object Parser::getObj(bool simpleOnly, const unsigned char *fileKey, CryptAlgori
         // dictionary or stream
     } else if (!simpleOnly && buf1.isCmd("<<")) {
         shift(objNum);
-        obj = Object(new Dict(lexer.getXRef()));
+        obj = Object(std::make_unique<Dict>(lexer.getXRef()));
         bool hasContentsEntry = false;
         while (!buf1.isCmd(">>") && !buf1.isEOF()) {
             if (!buf1.isName()) {
@@ -172,14 +172,14 @@ Object Parser::getObj(bool simpleOnly, const unsigned char *fileKey, CryptAlgori
         // stream objects are not allowed inside content streams or
         // object streams
         if (buf2.isCmd("stream")) {
-            if (allowStreams && (str = makeStream(std::move(obj), fileKey, encAlgorithm, keyLength, objNum, objGen, recursion + 1, strict))) {
-                return Object(std::unique_ptr<Stream>(str));
-            } else {
-                return Object::error();
+            if (allowStreams) {
+                if (auto str = makeStream(std::move(obj), fileKey, encAlgorithm, keyLength, objNum, objGen, recursion + 1, strict)) {
+                    return Object(std::move(str));
+                }
             }
-        } else {
-            shift();
+            return Object::error();
         }
+        shift();
 
         // indirect reference or integer
     } else if (buf1.isInt()) {
@@ -198,9 +198,8 @@ Object Parser::getObj(bool simpleOnly, const unsigned char *fileKey, CryptAlgori
             r.num = num;
             r.gen = gen;
             return Object(r);
-        } else {
-            return Object(num);
         }
+        return Object(num);
 
         // string
     } else if (decryptString && buf1.isString() && fileKey) {
@@ -223,10 +222,9 @@ err:
     return Object::error();
 }
 
-Stream *Parser::makeStream(Object &&dict, const unsigned char *fileKey, CryptAlgorithm encAlgorithm, int keyLength, int objNum, int objGen, int recursion, bool strict)
+std::unique_ptr<Stream> Parser::makeStream(Object &&dict, const unsigned char *fileKey, CryptAlgorithm encAlgorithm, int keyLength, int objNum, int objGen, int recursion, bool strict)
 {
     BaseStream *baseStr;
-    Stream *str;
     Goffset length;
     Goffset pos, endPos;
 
@@ -244,10 +242,11 @@ Stream *Parser::makeStream(Object &&dict, const unsigned char *fileKey, CryptAlg
 
     // get stream start position
     lexer.skipToNextLine();
-    if (!(str = lexer.getStream())) {
+    Stream *lexerStream;
+    if (!(lexerStream = lexer.getStream())) {
         return nullptr;
     }
-    pos = str->getPos();
+    pos = lexerStream->getPos();
 
     // get length
     Object obj = dict.dictLookup("Length", recursion);
@@ -316,15 +315,16 @@ Stream *Parser::makeStream(Object &&dict, const unsigned char *fileKey, CryptAlg
     }
 
     // make base stream
-    str = baseStr->makeSubStream(pos, true, length, std::move(dict)).release();
+    auto str = baseStr->makeSubStream(pos, true, length, std::move(dict));
 
     // handle decryption
     if (fileKey) {
-        str = new DecryptStream(str, fileKey, encAlgorithm, keyLength, { objNum, objGen });
+        str = std::make_unique<DecryptStream>(std::move(str), fileKey, encAlgorithm, keyLength, Ref { .num = objNum, .gen = objGen });
     }
 
     // get filters
-    str = str->addFilters(str->getDict(), recursion);
+    Dict *streamDict = str->getDict();
+    str = Stream::addFilters(std::move(str), streamDict, recursion);
 
     if (XRef *xref = lexer.getXRef()) {
         // Don't try to reuse the entry from the block at the start
