@@ -39,6 +39,7 @@
 // Copyright (C) 2024 Vincent Lefevre <vincent@vinc17.net>
 // Copyright (C) 2025 Arnav V <arnav0872@gmail.com>
 // Copyright (C) 2026 Ojas Maheshwari <workonlyojas@gmail.com>
+// Copyright (C) 2026 Adam Sampson <ats@offog.org>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -734,7 +735,7 @@ bool XRef::readXRefStream(Stream *xrefStr, Goffset *pos)
     }
 
     obj = dict->lookupNF("W").copy();
-    if (!obj.isArray() || obj.arrayGetLength() < 3) {
+    if (!obj.isArrayOfLengthAtLeast(3)) {
         return false;
     }
     for (int i = 0; i < 3; ++i) {
@@ -1357,7 +1358,7 @@ Object XRef::createDocInfoIfNeeded(Ref *ref)
 
     removeDocInfo();
 
-    obj = Object(new Dict(this));
+    obj = Object(std::make_unique<Dict>(this));
     *ref = addIndirectObject(obj);
     trailerDict.dictSet("Info", Object(*ref));
 
@@ -1530,10 +1531,10 @@ void XRef::removeIndirectObject(Ref r)
     setModified();
 }
 
-Ref XRef::addStreamObject(Dict *dict, std::vector<char> buffer, StreamCompression compression)
+Ref XRef::addStreamObject(std::unique_ptr<Dict> dict, std::vector<char> buffer, StreamCompression compression)
 {
     dict->add("Length", Object((int)buffer.size()));
-    auto stream = std::make_unique<AutoFreeMemStream>(std::move(buffer), Object(dict));
+    auto stream = std::make_unique<AutoFreeMemStream>(std::move(buffer), Object(std::move(dict)));
     stream->setFilterRemovalForbidden(true);
     switch (compression) {
     case StreamCompression::None:;
@@ -1660,7 +1661,7 @@ void XRef::XRefPreScanWriter::writeEntry(Goffset offset, int /*gen*/, XRefEntryT
 
 void XRef::writeStreamToBuffer(GooString *stmBuf, Dict *xrefDict, XRef *xref)
 {
-    auto *index = new Array(xref);
+    auto index = std::make_unique<Array>(xref);
     stmBuf->clear();
 
     // First pass: determine whether all offsets fit in 4 bytes or not
@@ -1669,16 +1670,16 @@ void XRef::writeStreamToBuffer(GooString *stmBuf, Dict *xrefDict, XRef *xref)
     const int offsetSize = prescan.hasOffsetsBeyond4GB ? sizeof(Goffset) : 4;
 
     // Second pass: actually write the xref stream
-    XRefStreamWriter writer(index, stmBuf, offsetSize);
+    XRefStreamWriter writer(index.get(), stmBuf, offsetSize);
     writeXRef(&writer, false);
 
     xrefDict->set("Type", Object(objName, "XRef"));
-    xrefDict->set("Index", Object(index));
-    auto *wArray = new Array(xref);
+    xrefDict->set("Index", Object(std::move(index)));
+    auto wArray = std::make_unique<Array>(xref);
     wArray->add(Object(1));
     wArray->add(Object(offsetSize));
     wArray->add(Object(2));
-    xrefDict->set("W", Object(wArray));
+    xrefDict->set("W", Object(std::move(wArray)));
 }
 
 bool XRef::parseEntry(Goffset offset, XRefEntry *entry)
